﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Collections;

namespace Clocks
{
    public partial class Form1 : Form
    {
        private ArrayList clocks;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            clocks = new ArrayList();
            RenderClocks();
        }

        private void RenderClocks()
        {
            foreach (ToolStripMenuItem t in contextMenuStrip1.Items)
            {
                if (t.Checked)
                {
                    Label l;
                    TimeZoneInfo tzi;

                    /* http://technet.microsoft.com/en-us/library/cc749073%28WS.10%29.aspx */
                    switch (t.Text)
                    {
                        case "Bragg":
                            l = Clock_Bragg;
                            tzi = TimeZoneInfo.FindSystemTimeZoneById("Eastern Standard Time");
                            break;
                        case "Iraq":
                            l = Clock_Iraq;
                            tzi = TimeZoneInfo.FindSystemTimeZoneById("Arabic Standard Time");
                            break;
                        case "Afghanistan":
                            l = Clock_Afghanistan;
                            /* Discovered it wasn't necessary to build a TZ for AF */
                            /*tzi = TimeZoneInfo.CreateCustomTimeZone("Afghanistan",
                                new TimeSpan(04, 30, 00), "(GMT+04:30) Afghanistan", "Afghanistan");*/
                            tzi = TimeZoneInfo.FindSystemTimeZoneById("Afghanistan Standard Time");
                            break;
                        case "Hawaii":
                            l = Clock_Hawaii;
                            tzi = TimeZoneInfo.FindSystemTimeZoneById("Hawaiian Standard Time");
                            break;
                        case "Korea":
                            l = Clock_Korea;
                            tzi = TimeZoneInfo.FindSystemTimeZoneById("Korea Standard Time");
                            break;
                        case "Zulu":
                            l = Clock_Zulu;
                            tzi = TimeZoneInfo.FindSystemTimeZoneById("UTC");
                            break;
                        default:
                            l = null;
                            tzi = null;
                            break;
                    }

                    if (l != null && tzi != null)
                    {
                        myClock c = new myClock(l, tzi, t.Text);
                        c.tClock.Start();
                        clocks.Add(c);
                    }
                }
            }
        }

        private void Form1_FormClosing(Object sender, FormClosingEventArgs e)
        {
            foreach (myClock c in clocks.ToArray())
            {
                c.closeThread();
            }

            // Just in case the threads didn't close on their own nicely...
            foreach (myClock c in clocks.ToArray())
            {
                c.tClock.Abort();
            }

            Application.Exit();
        }

        private void braggToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem t = (ToolStripMenuItem)sender;
            t.Checked = !t.Checked;
            Clock_Bragg.Visible = !Clock_Bragg.Visible;
        }

        private void iraqToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem t = (ToolStripMenuItem)sender;
            t.Checked = !t.Checked;
            Clock_Iraq.Visible = !Clock_Iraq.Visible;
        }

        private void afghanistanToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem t = (ToolStripMenuItem)sender;
            t.Checked = !t.Checked;
            Clock_Afghanistan.Visible = !Clock_Afghanistan.Visible;
        }

        private void hawaiiToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem t = (ToolStripMenuItem)sender;
            t.Checked = !t.Checked;
            Clock_Hawaii.Visible = !Clock_Hawaii.Visible;
        }

        private void koreaToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem t = (ToolStripMenuItem)sender;
            t.Checked = !t.Checked;
            Clock_Korea.Visible = !Clock_Korea.Visible;
        }

        private void Form1_MouseUp(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                contextMenuStrip1.Show(MousePosition);
            }
        }

        /* Had some trouble finding where to add this event handler.
         * For future reference, the damned thing was hidden because I had resized
         * the application window. */
        private void zuluToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem t = (ToolStripMenuItem)sender;
            t.Checked = !t.Checked;
            Clock_Zulu.Visible = !Clock_Zulu.Visible;
        }

        private void Clock_Bragg_Click(object sender, EventArgs e)
        {
            Clipboard.SetText(dtg(sender));
        }

        private void Clock_Iraq_Click(object sender, EventArgs e)
        {
            Clipboard.SetText(dtg(sender));
        }

        private void Clock_Afghanistan_Click(object sender, EventArgs e)
        {
            Clipboard.SetText(dtg(sender));
        }

        private void Clock_Hawaii_Click(object sender, EventArgs e)
        {
            Clipboard.SetText(dtg(sender));
        }

        private void Clock_Korea_Click(object sender, EventArgs e)
        {
            Clipboard.SetText(dtg(sender));
        }

        private void Clock_Zulu_Click(object sender, EventArgs e)
        {
            Clipboard.SetText(dtg(sender));
        }

        private string dtg (object s)
        {
            foreach (myClock c in clocks)
            {
                if (c.l.Equals(s))
                {
                    DateTime d = DateTime.Now;
                    d = TimeZoneInfo.ConvertTime(d, TimeZoneInfo.Local, c.tzi);
                    return (d.ToString("ddHHmmMMMyy"));
                }
            }
            // if you make it to here something went wrong
            return ("");
        }

        // This block of code is kind of stupid but it's there - both the single and double
        // click events are handled by this redundant redundancy.
        private void Clock_Bragg_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            Clock_Bragg_Click(sender, e);
        }

        private void Clock_Iraq_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            Clock_Iraq_Click(sender, e);
        }

        private void Clock_Afghanistan_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            Clock_Afghanistan_Click(sender, e);
        }

        private void Clock_Hawaii_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            Clock_Hawaii_Click(sender, e);
        }

        private void Clock_Korea_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            Clock_Korea_Click(sender, e);
        }

        private void Clock_Zulu_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            Clock_Zulu_Click(sender, e);
        }
    }

    public class myClock
    {
        public Label l;
        public TimeZoneInfo tzi;
        private bool closing;
        private string location;

        public Thread tClock;

        public myClock(Label myLabel, TimeZoneInfo tzi, string location)
        {
            this.l = myLabel;
            this.tzi = tzi;
            this.tClock = new Thread(this.runClock);
            this.closing = false;
            this.location = location;
        }

        private void SetText(string text)
        {
            this.l.Text = text;
        }

        public void closeThread()
        {
            this.closing = true;
        }

        public delegate void SetTextCallback(string text);

        public void runClock()
        {
            DateTime thisTime;

            while (!closing)
            {
                thisTime = DateTime.Now;
                thisTime = TimeZoneInfo.ConvertTime(thisTime, TimeZoneInfo.Local, tzi);

                if (l.InvokeRequired)
                {
                    this.l.Invoke (new SetTextCallback(this.SetText),
                        new object[] { string.Format("{0} {1:d2}:{2:d2}",
                            this.location, thisTime.Hour, thisTime.Minute) });
                }
                else
                {
                    SetText(thisTime.Hour + ":" + thisTime.Minute);
                }
                Thread.Sleep(1000);
            }
        }
    }
}
